@@ -32,7 +32,7 @@
 #include <time.h>
 
 #include <libedataserver/e-sexp.h>
-#include <e-util/e-uid.h>
+#include <libedataserver/e-uid.h>
 #include <e-util/e-util.h>
 #include <libebook/e-address-western.h>
 #include <libebook/e-contact.h>
@@ -118,7 +118,7 @@ http_status_to_pas (E2kHTTPStatus status)
 
 	switch (status) {
 	case E2K_HTTP_UNAUTHORIZED:
-		return GNOME_Evolution_Addressbook_AuthenticationFailed;
+		return GNOME_Evolution_Addressbook_PermissionDenied;
 	case E2K_HTTP_CANT_CONNECT:
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 	default:
@@ -485,7 +485,6 @@ build_cache (EBookBackendExchange *be)
 	e_book_backend_cache_set_populated (bepriv->cache);
 	bepriv->is_cache_ready=TRUE;
 	e_file_cache_thaw_changes (E_FILE_CACHE (bepriv->cache));
-
 	return NULL;
 }
 
@@ -1853,7 +1852,7 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 
 	d(printf("ebbe_start_book_view(%p, %p)\n", backend, book_view));
 
-	g_object_ref (book_view);
+	bonobo_object_ref (book_view);
 	e_data_book_view_notify_status_message (book_view, _("Searching..."));
 
 	switch (bepriv->mode) {
@@ -1894,6 +1893,14 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 		
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
 
+		if (!be->priv->ctx) {
+			e_book_backend_notify_auth_required (backend);
+			e_data_book_view_notify_complete (book_view,
+						GNOME_Evolution_Addressbook_AuthenticationRequired);
+			bonobo_object_unref (book_view);
+			return;
+		}
+
 		/* execute the query */
 		rn = e_book_backend_exchange_build_restriction (query, 
 							bepriv->base_rn);
@@ -1902,7 +1909,7 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 					       field_names, n_field_names,
 					       rn, NULL, TRUE);
 		e2k_restriction_unref (rn);
-
+		
 		while ((result = e2k_result_iter_next (iter))) {
 			contact = e_contact_from_props (be, result);
 			if (contact) {
@@ -1915,7 +1922,7 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 
 		e_data_book_view_notify_complete (book_view,
 						  http_status_to_pas (status));
-		g_object_unref (book_view);
+		bonobo_object_unref (book_view);
 
 	default:
 		break;
@@ -2134,7 +2141,7 @@ e_book_backend_exchange_authenticate_user (EBookBackend *backend,
 				return;
 			}
 		}
-		if (!bepriv->connected);
+		if (!bepriv->connected)
 			e_book_backend_exchange_connect (be);
 		if (e_book_backend_cache_is_populated (bepriv->cache)) {
 			if (bepriv->is_writable)
@@ -2259,8 +2266,6 @@ e_book_backend_exchange_load_source (EBookBackend *backend,
 		e_book_backend_set_is_writable (backend, FALSE);
 		e_book_backend_notify_writable (backend, FALSE);
 		e_book_backend_notify_connection_status (backend, FALSE);
-	}
-	if (bepriv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
 		if (!e_book_backend_cache_exists (bepriv->original_uri))
 			return GNOME_Evolution_Addressbook_OfflineUnavailable;
 	}
@@ -2271,7 +2276,10 @@ e_book_backend_exchange_load_source (EBookBackend *backend,
 		return GNOME_Evolution_Addressbook_Success;
 	}
 
+	// writable property will be set in authenticate_user callback
+	e_book_backend_set_is_writable (E_BOOK_BACKEND(backend), FALSE);
 	e_book_backend_set_is_loaded (E_BOOK_BACKEND (be), TRUE);
+	e_book_backend_notify_connection_status (E_BOOK_BACKEND (be), TRUE);
 
 	if (e_book_backend_cache_is_populated (bepriv->cache)) {
 		if (bepriv->is_writable)
@@ -2297,28 +2305,18 @@ e_book_backend_exchange_remove (EBookBackendSync *backend, EDataBook *book, guin
 	d(printf("ebbe_remove(%p, %p)\n", backend, book));
 	uri = e_folder_exchange_get_internal_uri (bepriv->folder);
 	result = exchange_account_remove_folder (bepriv->account, uri);
-	return GNOME_Evolution_Addressbook_Success;
-	/* FIXME: Folder deletion from contacts view */
-#if 0 
-	EBookBackendExchange *be = E_BOOK_BACKEND_EXCHANGE (backend);
-	E2kHTTPStatus status;
-
-	/*
-	char *path;
-	path = strstr (be->priv->exchange_uri, "://");
-	if (path)
-		path = strchr (path + 3, '/');
-	status = exchange_account_remove_folder (be->priv->account, path);
-	g_free (path);
-	 */
-	status = e_folder_exchange_delete (be->priv->folder, NULL);
-	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status))
+	if (result == EXCHANGE_ACCOUNT_FOLDER_OK)
 		return GNOME_Evolution_Addressbook_Success;
-	else if (status == E2K_HTTP_UNAUTHORIZED)
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST)
+		return GNOME_Evolution_Addressbook_NoSuchBook;
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_UNSUPPORTED_OPERATION)
+		return GNOME_Evolution_Addressbook_PermissionDenied;
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_OFFLINE)
+		return GNOME_Evolution_Addressbook_OfflineUnavailable;
+	else if (result == EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED)
 		return GNOME_Evolution_Addressbook_PermissionDenied;
 	else
 		return GNOME_Evolution_Addressbook_OtherError;
-#endif
 }
 
 static char *
@@ -2348,20 +2346,20 @@ e_book_backend_exchange_set_mode (EBookBackend *backend, int mode)
 
 	bepriv->mode = mode;
 	/* if (e_book_backend_is_loaded (backend)) { */
-		if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
-			e_book_backend_set_is_writable (backend, FALSE);
-			e_book_backend_notify_writable (backend, FALSE);
-			e_book_backend_notify_connection_status (backend, FALSE);
-			/* FIXME : free context ? */
-		} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
-			e_book_backend_set_is_writable (backend, bepriv->is_writable);
-			e_book_backend_notify_writable (backend, bepriv->is_writable);
-			e_book_backend_notify_connection_status (backend, TRUE);
-			account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
-			if (!exchange_account_get_context (account))
-				e_book_backend_notify_auth_required (backend);
-		}
-/*	} */
+	if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
+		e_book_backend_set_is_writable (backend, FALSE);
+		e_book_backend_notify_writable (backend, FALSE);
+		e_book_backend_notify_connection_status (backend, FALSE);
+		/* FIXME : free context ? */
+	} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
+		e_book_backend_set_is_writable (backend, bepriv->is_writable);
+		e_book_backend_notify_writable (backend, bepriv->is_writable);
+		e_book_backend_notify_connection_status (backend, TRUE);
+		account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+		if (!exchange_account_get_context (account))
+			e_book_backend_notify_auth_required (backend);
+	}
+	/*}*/
 }
 
 /**
